%% The newspaper example
close all; clear all
img = imread('newspaper.png');
figure; colormap(gray(256)); imagesc(img); axis equal; axis off
%% Fourier domain
factor = 10;
F = fft2(img);
T = log(abs(F(1,1)))-2;
imagesc(log(fftshift(abs(F))),[5,T]); axis equal; axis off
s = size(img)/2;
x = [-1,1,1,-1,-1]*round(s(1)/factor) + s(1) + 1;
y = [-1,-1,1,1,-1]*round(s(2)/factor) + s(2) + 1;
hold on, plot(y,x,'r')
%% Downsample directly
bad = img(1:factor:end,1:factor:end);
clf; imagesc(bad); axis equal; axis off
%% Fourier domain
G = fft2(bad);
imagesc(log(fftshift(abs(G)))); axis equal; axis off
%% Downsample by cropping Fourier domain
G = fftshift(F);
G = G(x(1):x(2),y(2):y(3));
G = ifftshift(G);
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse transform
g = real(ifft2(G));
imagesc(g); axis equal; axis off
%% Alternative: low-pass filtering (ideal)
H = zeros(size(img));
H(x(1):x(2),y(2):y(3)) = 1;
imagesc(H); axis equal; axis off
H = ifftshift(H);
hold on, plot(y,x,'r')
%% Filter
G = F.*H;
clf, imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
hold on, plot(y,x,'r')
%% Inverse Fourier transform
g = ifft2(G);
clf; imagesc(g); axis equal; axis off
%% Downsample low-pass filtered image
good = g(1:factor:end,1:factor:end);
imagesc(good); axis equal; axis off
%% Spatial filter
h = ifft2(H);
h = ifftshift(h); h = h(size(h,1)/2+[-99:100],size(h,2)/2+[-99:100]);
imagesc(h); axis equal; axis off
%% Alternative: low-pass filtering (Gaussian)
u = size(img)/2;
s = u/factor/2;
[u,v] = ndgrid(-u(1):u(1)-1,-u(2):u(2)-1);
H = exp ( -( (u/s(1)).^2+(v/s(2)).^2 ) / 2 );
imagesc(H); axis equal; axis off
H = ifftshift(H);
hold on, plot(y,x,'r')
%% Filter
G = F.*H;
clf, imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
hold on, plot(y,x,'r')
%% Inverse Fourier transform
g = ifft2(G);
clf; imagesc(g); axis equal; axis off
%% Downsample low-pass filtered image
good = g(1:factor:end,1:factor:end);
imagesc(good); axis equal; axis off
%% Spatial filter
h = ifft2(H);
h = ifftshift(h); h = h(size(h,1)/2+[-99:100],size(h,2)/2+[-99:100]);
imagesc(h); axis equal; axis off
