%% An image
close all; clear all
f = imread('cameraman.tif');
figure; colormap(gray(256)); imagesc(f); axis equal; axis off
%% Fourier transform
F = fft2(f);
T = log(abs(F(1,1)));
imagesc(log(fftshift(abs(F))),[5,T]); axis equal; axis off
%% Low-pass filter
u = size(f)/2;
[u,v] = ndgrid(-u(1):u(1)-1,-u(2):u(2)-1);
H = u.^2+v.^2 < 100.^2;
imagesc(H); axis equal; axis off
H = ifftshift(H);
%% Filter
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off
%% Again!
H = u.^2+v.^2 < 50.^2;
H = ifftshift(H);
G = F.*H;
g = ifft2(G);
imagesc(g); axis equal; axis off
%% Again!
H = u.^2+v.^2 < 20.^2;
H = ifftshift(H);
G = F.*H;
g = ifft2(G);
imagesc(g); axis equal; axis off
%% What does the spatial filter look like?
h = ifft2(H);
imagesc(fftshift(h)); axis equal; axis off

%% Anisotropic low-pass filter
H = (u/5).^2+v.^2 < 20.^2;
H = ifftshift(H);
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off

%% High-pass filter
H = u.^2+v.^2 > 20.^2;
imagesc(H); axis equal; axis off
H = ifftshift(H);
%% Filter
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off

%% Band-pass filter
H = ( u.^2+v.^2 > 20.^2 ) & u.^2+v.^2 < 30.^2;
imagesc(H); axis equal; axis off
H = ifftshift(H);
%% Filter
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off

%%%%%%%%%%%%

%% Smoother filters in the Fourier domain
H = exp ( -( u.^2+v.^2 ) / (2*20^2) );
imagesc(H); axis equal; axis off
H = ifftshift(H);
%% Filter
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off

%%%%%%%%%%%%

%% Let's add some pattern to the image
close all; clear all
f = imread('cameraman.tif');
x = size(f);
[x,y] = ndgrid(0:x(1)-1,0:x(2)-1);
u = 2*pi/256*25;
v = 2*pi/256*20;
f = double(f)*0.75 + 30*(1+cos(x*u+y*v));
figure; colormap(gray(256)); imagesc(f); axis equal; axis off
%% Fourier transform
F = fft2(f);
T = log(abs(F(1,1)));
imagesc(log(fftshift(abs(F))),[5,T]); axis equal; axis off
%% Notch filter
H = ones(size(f));
H(129+round(u*128/pi)+(-3:3),129+round(v*128/pi)+(-3:3)) = 0;
H(129-round(u*128/pi)+(-3:3),129-round(v*128/pi)+(-3:3)) = 0;
imagesc(H); axis equal; axis off
H = ifftshift(H);
%% Filter
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off
%% What does the spatial filter look like?
h = ifft2(H);
h(1,1) = 0; % blanking out the central high-intensity peak
imagesc(fftshift(h)); axis equal; axis off
%% Gaussian notch filter
H = ones(size(f));
g = exp(-(-9:9).^2/(2*3^2)); g = g'*g;
H(129+round(u*128/pi)+(-9:9),129+round(v*128/pi)+(-9:9)) = 1-g;
H(129-round(u*128/pi)+(-9:9),129-round(v*128/pi)+(-9:9)) = 1-g;
imagesc(H); axis equal; axis off
H = ifftshift(H);
%% Filter
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off
%% What does the spatial filter look like?
h = ifft2(H);
h(1,1) = 0; % blanking out the central high-intensity peak
imagesc(fftshift(h)); axis equal; axis off
%% Larger Gaussian notch filter
H = ones(size(f));
g = exp(-(-23:23).^2/(2*9^2)); g = g'*g;
H(129+round(u*128/pi)+(-23:23),129+round(v*128/pi)+(-23:23)) = 1-g;
H(129-round(u*128/pi)+(-23:23),129-round(v*128/pi)+(-23:23)) = 1-g;
imagesc(H); axis equal; axis off
H = ifftshift(H);
%% Filter
G = F.*H;
imagesc(log(fftshift(abs(G))),[5,T]); axis equal; axis off
%% Inverse Fourier transform
g = ifft2(G);
imagesc(g); axis equal; axis off
%% What does the spatial filter look like?
h = ifft2(H);
h(1,1) = 0; % blanking out the central high-intensity peak
imagesc(fftshift(h)); axis equal; axis off
